#ifndef distinst_h
#define distinst_h

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>

/*
 * Log level
 */
typedef enum {
  DISTINST_LOG_LEVEL_TRACE = 0,
  DISTINST_LOG_LEVEL_DEBUG = 1,
  DISTINST_LOG_LEVEL_INFO = 2,
  DISTINST_LOG_LEVEL_WARN = 3,
  DISTINST_LOG_LEVEL_ERROR = 4,
} DISTINST_LOG_LEVEL;

/*
 * Bootloader steps
 */
typedef enum {
  DISTINST_STEP_INIT = 0,
  DISTINST_STEP_PARTITION = 1,
  DISTINST_STEP_FORMAT = 2,
  DISTINST_STEP_EXTRACT = 3,
  DISTINST_STEP_CONFIGURE = 4,
  DISTINST_STEP_BOOTLOADER = 5,
} DISTINST_STEP;

/*
 * An installer object
 */
typedef struct {

} DistinstInstaller;

/*
 * Installer error message
 */
typedef struct {
  DISTINST_STEP step;
  int err;
} DistinstError;

/*
 * Installer status message
 */
typedef struct {
  DISTINST_STEP step;
  int percent;
} DistinstStatus;

/*
 * Installer configuration
 */
typedef struct {
  const char *squashfs;
  const char *disk;
} DistinstConfig;

/*
 * Installer error callback
 */
typedef void (*DistinstErrorCallback)(const DistinstError*, void*);

/*
 * Installer status callback
 */
typedef void (*DistinstStatusCallback)(const DistinstStatus*, void*);

/*
 * Installer log callback
 */
typedef void (*DistinstLogCallback)(DISTINST_LOG_LEVEL, const char*, void*);

/*
 * Destroy an installer object
 */
void distinst_installer_destroy(DistinstInstaller *installer);

/*
 * Send an installer status message
 */
void distinst_installer_emit_error(DistinstInstaller *installer, const DistinstError *error);

/*
 * Send an installer status message
 */
void distinst_installer_emit_status(DistinstInstaller *installer, const DistinstStatus *status);

/*
 * Install using this installer
 */
int distinst_installer_install(DistinstInstaller *installer, const DistinstConfig *config);

/*
 * Create an installer object
 */
DistinstInstaller *distinst_installer_new();

/*
 * Set the installer status callback
 */
void distinst_installer_on_error(DistinstInstaller *installer,
                                 DistinstErrorCallback callback,
                                 void *user_data);

/*
 * Set the installer status callback
 */
void distinst_installer_on_status(DistinstInstaller *installer,
                                  DistinstStatusCallback callback,
                                  void *user_data);

/*
 * Initialize logging
 */
int distinst_log(DistinstLogCallback callback, void *user_data);

#endif /* distinst_h */
